/*
	Trigger definition
	(c) Copyright 1992/1994 Marco Savegnago IW3FQG

	TTriggerItem serve da classe base per tutti gli oggetti che concorrono
	alla generazione di allarmi.
	Es: un LogicalInput oltre che essere un TInput e' anche un
	possibile generatore di allarmi.
*/

#ifndef	__TRIGITEM_H
#define	__TRIGITEM_H

#include "tbobject.h"

struct	TTriggerItem
{
	/*
		Gli item che e' possibile far concorrere a generare un allarme
		sono:

		-	LogicalInput
		-	AnalogInput
		-	LogicalOutput
		-	AnalogOutput
		-	Timer

		-	AllarmEvent
		-	CDROM

	*/

	enum IType
	{
		LogicInput		=	0x0001,
		AnalogInput		=	0x0002,
		LogicOutput		=	0x0004,
		AnalogOutput	=	0x0008,
		Timer			=	0x0010,
		AlarmEvent		=	0x0020,
		CDROM			=	0x0040,
		EventFlag		=	0x0080,
		CounterFlag		=	0x0100
	} iType;

	// Il costruttore

	TTriggerItem(TTriggerItem::IType iType_)
	:
		iType(iType_)
		{
		}
	/*
		Le  due funzioni che ogni alarmitem deve obbligatoriamente
		ridefinire per poter essere considerato tale
	*/

	virtual	Boolean Is() = 0;
	virtual	Boolean IsNot() = 0;
	virtual	void	Reset() {};
	virtual	Boolean	IsEq(int value) {	return False; }
};

/*
	AlarmEvent e' la classe che genera un allarme
	Accetta come selettori delle condizioni degli alarmItem e vengono
	testate quando necessario nell'applicazione.
	Si occupano automaticamente di risolvere e verificare le condizioni
*/

class TTriggerEvent
:
	public TTriggerItem
{

protected:

	//	Stato dell'allarm event

	Boolean	active;

public:
	//	Lo stato per cui e' verificata la condizione SE E' o SE NON E'

	enum State          { NOSTAT, IS, ISNOT, ISEQUAL };

	//	Il tipo di condizione

	enum Conditional	{ NOCOND, IF, IFNOT, ELSEIF, ELSEIFNOT, ENDIF };

	//	Il legame logico tra due elementi

	enum Logical        { NOLOGI, AND, OR };
protected:

	int nestedIf;					// Numero di If nidificati

	/*	Implementazione dell'item	*/

	struct	TTriggerItemImp
	{
		Conditional		conditional;	// IF, ELSEIF, ENDIF
		State			state;			// ISNOT, IS
		union
		{
			int		iValue;
			Boolean bValue;
		}	value;

		Logical			relation; 		// AND, OR, NIENTE

		TTriggerItem*		item;

		TTriggerItemImp()
		:
			conditional(IF),
			state(IS),
			relation(NOLOGI),
			item(0)
			{
				value.iValue=0;
			}

		TTriggerItemImp(Conditional _conditional, State _state, TTriggerItem* _item, Logical _relation )
		:
			conditional(_conditional),
			state(_state),
			relation(_relation),
			item(_item)
			{
				value.iValue=0;
			}

		TTriggerItemImp(Conditional _conditional, State _state, TTriggerItem* _item, int _value, Logical _relation )
		:
			conditional(_conditional),
			state(_state),
			relation(_relation),
			item(_item)
			{
				value.iValue	=	_value;
			}

		Boolean Is()
		{
			return item->Is();
		}

		Boolean IsNot()
		{
			return item->IsNot();
		}

		void	Reset()
		{
			item->Reset();
		}

		Boolean IsEqual(int value)
		{
			return item->IsEq(value);
		}

		Boolean	isATimer()
		{
			return Boolean(item->iType == TTriggerItem::Timer);
		}
	};

	TBNSCollection	*items;

	char*	szname;
public:

	TTriggerEvent(const char* name="");					// Costruttore di default
	virtual	~TTriggerEvent();			// distruttore


public:
	TTriggerEvent& If(Logical operand = NOLOGI);			// Imposta inizio delle condizioni
	TTriggerEvent& If(TTriggerItem*, Logical operand = NOLOGI);	// Imposta inizio delle condizioni

	TTriggerEvent& Endif(Logical operand = NOLOGI);		// Imposta la fine delle condizioni

	TTriggerEvent& And();									// Associa come AND logico
	TTriggerEvent& Or();									// Associa come OR 	logico

	TTriggerEvent& Is(TTriggerItem*, Logical operand = NOLOGI);	// Imposta confronto per vero
	TTriggerEvent& IsEqual(TTriggerItem* item, int value, Logical operand = NOLOGI);	// Imposta confronto per uguale
	TTriggerEvent& IsNot(TTriggerItem*, Logical operand = NOLOGI);// Imposta confronto per falso

	void   		reset();								// Resetta le condizioni di trigger

	Boolean 	compute(int level, TTriggerItemImp*	item);  // Risolve i vari livello dell'espressione
	Boolean 	resolve();							// Risolve l'intera esperessione

	Boolean		isActive();
	void		activate(Boolean act = True);
	char*		GetName();

protected:

	TTriggerItemImp*	skip(int level, TTriggerItemImp*	item);		// Salta condizioni da non testare

	void	resetTimers();
public:
	virtual	Boolean Is();
	virtual	Boolean IsNot();
};

inline Boolean TTriggerEvent::Is()
{
	return Boolean(True == resolve());
}

inline	Boolean TTriggerEvent::IsNot()
{
	return Boolean (False == resolve());
}

inline 	Boolean  TTriggerEvent::isActive()
{
	return active;
}

inline  void	TTriggerEvent::activate(Boolean act)
{
	active = act;
}

inline	char*	TTriggerEvent::GetName()
{
	return	szname;
}

#endif	__TRIGITEM_H
